@@ -21,7 +21,7 @@
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
-__version__ = "1.8.6"
+__version__ = "1.8.8"
 
 __all__ = [
 
@@ -97,17 +97,23 @@ class MalformedUrl(Exception):
 #######################################################################
 #######################################################################
 
+def plain(*args):
+    bb.msg.warn(''.join(args))
+
 def debug(lvl, *args):
-    bb.msg.std_debug(lvl, ''.join(args))
+    bb.msg.debug(lvl, None, ''.join(args))
 
 def note(*args):
-    bb.msg.std_note(''.join(args))
+    bb.msg.note(1, None, ''.join(args))
+
+def warn(*args):
+    bb.msg.warn(1, None, ''.join(args))
 
 def error(*args):
-    bb.msg.std_error(''.join(args))
+    bb.msg.error(None, ''.join(args))
 
 def fatal(*args):
-    bb.msg.std_fatal(''.join(args))
+    bb.msg.fatal(None, ''.join(args))
 
 
 #######################################################################
@@ -345,14 +351,20 @@ def encodeurl(decoded):
 #######################################################################
 
 def which(path, item, direction = 0):
-    """Useful function for locating a file in a PATH"""
-    found = ""
+    """
+    Locate a file in a PATH
+    """
+
+    paths = (path or "").split(':')
+    if direction != 0:
+        paths.reverse()
+
     for p in (path or "").split(':'):
-        if os.path.exists(os.path.join(p, item)):
-            found = os.path.join(p, item)
-            if direction == 0:
-                break
-    return found
+        next = os.path.join(p, item)
+        if os.path.exists(next):
+            return next
+
+    return ""
 
 #######################################################################
 
@@ -1112,7 +1124,12 @@ class digraph:
 
     def allnodes(self):
         "returns all nodes in the dictionary"
-        return self.dict.keys()
+        keys = self.dict.keys()
+        ret = []
+        for key in keys:
+            ret.append(key)
+        ret.sort()
+        return ret
 
     def firstzero(self):
         "returns first node with zero references, or NULL if no such node exists"
@@ -1156,7 +1173,12 @@ class digraph:
     def getparents(self, item):
         if not self.hasnode(item):
             return []
-        return self.dict[item][1]
+        parents = self.dict[item][1]
+        ret = []
+        for parent in parents:
+            ret.append(parent)
+        ret.sort()
+        return ret
 
     def getchildren(self, item):
         if not self.hasnode(item):
