@@ -73,6 +73,19 @@ class BBCooker:
         self.configuration.event_data = bb.data.createCopy(self.configuration.data)
         bb.data.update_data(self.configuration.event_data)
 
+        #
+        # TOSTOP must not be set or our children will hang when they output
+        #
+        fd = sys.stdout.fileno()
+        if os.isatty(fd):
+            import termios
+            tcattr = termios.tcgetattr(fd)
+            if tcattr[3] & termios.TOSTOP:
+                bb.msg.note(1, bb.msg.domain.Build, "The terminal had the TOSTOP bit set, clearing...")
+                tcattr[3] = tcattr[3] & ~termios.TOSTOP
+                termios.tcsetattr(fd, termios.TCSANOW, tcattr)
+
+
     def tryBuildPackage(self, fn, item, task, the_data, build_depends):
         """
         Build one task of a package, optionally build following task depends
@@ -143,10 +156,11 @@ class BBCooker:
         if self.configuration.buildfile:
             self.cb = None
             self.bb_cache = bb.cache.init(self)
+            bf = self.matchFile(self.configuration.buildfile)
             try:
-                self.configuration.data = self.bb_cache.loadDataFull(self.configuration.buildfile, self.configuration.data)
+                self.configuration.data = self.bb_cache.loadDataFull(bf, self.configuration.data)
             except IOError, e:
-                bb.msg.fatal(bb.msg.domain.Parsing, "Unable to read %s: %s" % ( self.configuration.buildfile, e ))
+                bb.msg.fatal(bb.msg.domain.Parsing, "Unable to read %s: %s" % (bf, e))
             except Exception, e:
                 bb.msg.fatal(bb.msg.domain.Parsing, "%s" % e)
         # emit variables and shell functions
@@ -336,6 +350,10 @@ class BBCooker:
                 if bb.data.getVarFlag(var, 'handler', data):
                     bb.event.register(var,bb.data.getVar(var, data))
 
+            bb.fetch.fetcher_init(self.configuration.data)
+
+            bb.event.fire(bb.event.ConfigParsed(self.configuration.data))
+
         except IOError:
             bb.msg.fatal(bb.msg.domain.Parsing, "Unable to open %s" % afile )
         except bb.parse.ParseError, details:
@@ -373,14 +391,15 @@ class BBCooker:
             bb.data.setVar("BUILDNAME", os.popen('date +%Y%m%d%H%M').readline().strip(), self.configuration.data)
         bb.data.setVar("BUILDSTART", time.strftime('%m/%d/%Y %H:%M:%S',time.gmtime()),self.configuration.data)
 
-    def buildFile(self, buildfile):
+    def matchFile(self, buildfile):
         """
-        Build the file matching regexp buildfile
+        Convert the fragment buildfile into a real file
+        Error if there are too many matches
         """
-
         bf = os.path.abspath(buildfile)
         try:
             os.stat(bf)
+            return bf
         except OSError:
             (filelist, masked) = self.collect_bbfiles()
             regexp = re.compile(buildfile)
@@ -394,7 +413,14 @@ class BBCooker:
                 for f in matches:
                     bb.msg.error(bb.msg.domain.Parsing, "    %s" % f)
                 sys.exit(1)
-            bf = matches[0]		    
+            return matches[0]		    
+
+    def buildFile(self, buildfile):
+        """
+        Build the file matching regexp buildfile
+        """
+
+        bf = self.matchFile(buildfile)
 
         bbfile_data = bb.parse.handle(bf, self.configuration.data)
 
@@ -597,7 +623,7 @@ class BBCooker:
         return (finalfiles, masked)
 
     def parse_bbfiles(self, filelist, masked, progressCallback = None):
-        parsed, cached, skipped = 0, 0, 0
+        parsed, cached, skipped, error = 0, 0, 0, 0
         for i in xrange( len( filelist ) ):
             f = filelist[i]
 
@@ -640,6 +666,7 @@ class BBCooker:
                 self.bb_cache.sync()
                 raise
             except Exception, e:
+                error += 1
                 self.bb_cache.remove(f)
                 bb.msg.error(bb.msg.domain.Collection, "%s while parsing %s" % (e, f))
             except:
@@ -651,3 +678,6 @@ class BBCooker:
             bb.msg.note(1, bb.msg.domain.Collection, "Parsing finished. %d cached, %d parsed, %d skipped, %d masked." % ( cached, parsed, skipped, masked ))
 
         self.bb_cache.sync()
+
+        if error > 0:
+            bb.msg.fatal(bb.msg.domain.Collection, "Parsing errors found, exiting...")
