@@ -176,15 +176,23 @@ def handle(fn, d, include = 0):
                 if bb.event.register(var,handler) == bb.event.Registered:
                     all_handlers[var] = handler
 
+            tasklist = {}
             for var in data.getVar('__BBTASKS', d) or []:
+                if var not in tasklist:
+                    tasklist[var] = []
                 deps = data.getVarFlag(var, 'deps', d) or []
+                for p in deps:
+                    if p not in tasklist[var]:
+                        tasklist[var].append(p)
+
                 postdeps = data.getVarFlag(var, 'postdeps', d) or []
-                bb.build.add_task(var, deps, d)
                 for p in postdeps:
-                    pdeps = data.getVarFlag(p, 'deps', d) or []
-                    pdeps.append(var)
-                    data.setVarFlag(p, 'deps', pdeps, d)
-                    bb.build.add_task(p, pdeps, d)
+                    if p not in tasklist:
+                        tasklist[p] = []
+                    if var not in tasklist[p]:
+                        tasklist[p].append(var)
+
+            bb.build.add_tasks(tasklist, d)
 
             # now add the handlers
             if not len(all_handlers) == 0:
@@ -390,25 +398,27 @@ def vars_from_file(mypkg, d):
 def set_additional_vars(file, d, include):
     """Deduce rest of variables, e.g. ${A} out of ${SRC_URI}"""
 
-    bb.msg.debug(2, bb.msg.domain.Parsing, "BB %s: set_additional_vars" % file)
+    return
+    # Nothing seems to use this variable
+    #bb.msg.debug(2, bb.msg.domain.Parsing, "BB %s: set_additional_vars" % file)
 
-    src_uri = data.getVar('SRC_URI', d, 1)
-    if not src_uri:
-        return
+    #src_uri = data.getVar('SRC_URI', d, 1)
+    #if not src_uri:
+    #    return
 
-    a = (data.getVar('A', d, 1) or '').split()
+    #a = (data.getVar('A', d, 1) or '').split()
 
-    from bb import fetch
-    try:
-        fetch.init(src_uri.split(), d)
-    except fetch.NoMethodError:
-        pass
-    except bb.MalformedUrl,e:
-        raise ParseError("Unable to generate local paths for SRC_URI due to malformed uri: %s" % e)
+    #from bb import fetch
+    #try:
+    #    ud = fetch.init(src_uri.split(), d)
+    #    a += fetch.localpaths(d, ud)
+    #except fetch.NoMethodError:
+    #    pass
+    #except bb.MalformedUrl,e:
+    #    raise ParseError("Unable to generate local paths for SRC_URI due to malformed uri: %s" % e)
+    #del fetch
 
-    a += fetch.localpaths(d)
-    del fetch
-    data.setVar('A', " ".join(a), d)
+    #data.setVar('A', " ".join(a), d)
 
 
 # Add us to the handlers list
