@@ -45,14 +45,17 @@ def localpath(fn, d):
     if os.path.exists(fn):
         return fn
 
+    if "://" not in fn:
+        return fn
+
     localfn = None
     try:
-        localfn = bb.fetch.localpath(fn, d)
+        localfn = bb.fetch.localpath(fn, d, False)
     except bb.MalformedUrl:
         pass
 
     if not localfn:
-        localfn = fn
+        return fn
     return localfn
 
 def obtain(fn, data):
@@ -67,7 +70,7 @@ def obtain(fn, data):
             return localfn
         bb.mkdirhier(dldir)
         try:
-            bb.fetch.init([fn])
+            bb.fetch.init([fn], data)
         except bb.fetch.NoMethodError:
             (type, value, traceback) = sys.exc_info()
             bb.msg.debug(1, bb.msg.domain.Parsing, "obtain: no method: %s" % value)
@@ -178,7 +181,9 @@ def feeder(lineno, s, fn, data):
             if val == None:
                 val = groupd["value"]
         elif "colon" in groupd and groupd["colon"] != None:
-            val = bb.data.expand(groupd["value"], data)
+            e = data.createCopy()
+            bb.data.update_data(e)
+            val = bb.data.expand(groupd["value"], e)
         elif "append" in groupd and groupd["append"] != None:
             val = "%s %s" % ((getFunc(groupd, key, data) or ""), groupd["value"])
         elif "prepend" in groupd and groupd["prepend"] != None:
