@@ -22,7 +22,7 @@ Handles preparation and execution of a queue of tasks
 # with this program; if not, write to the Free Software Foundation, Inc.,
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
-from bb import msg, data, fetch, event, mkdirhier, utils
+from bb import msg, data, event, mkdirhier, utils
 from sets import Set 
 import bb, os, sys
 import signal
@@ -95,7 +95,7 @@ class RunQueue:
             # Nothing to do
             return
 
-        bb.msg.note(1, bb.msg.domain.RunQueue, "Preparing Runqueue")
+        bb.msg.note(1, bb.msg.domain.RunQueue, "Preparing runqueue")
 
         for task in range(len(taskData.tasks_name)):
             fnid = taskData.tasks_fnid[task]
@@ -110,9 +110,10 @@ class RunQueue:
                 if 'deptask' in task_deps and taskData.tasks_name[task] in task_deps['deptask']:
                     taskname = task_deps['deptask'][taskData.tasks_name[task]]
                     for depid in taskData.depids[fnid]:
+                        # Won't be in build_targets if ASSUME_PROVIDED
                         if depid in taskData.build_targets:
                             depdata = taskData.build_targets[depid][0]
-                            if depdata:
+                            if depdata is not None:
                                 dep = taskData.fn_index[depdata]
                                 depends.append(taskData.gettask_id(dep, taskname))
 
@@ -122,7 +123,7 @@ class RunQueue:
                     for depid in taskData.rdepids[fnid]:
                         if depid in taskData.run_targets:
                             depdata = taskData.run_targets[depid][0]
-                            if depdata:
+                            if depdata is not None:
                                 dep = taskData.fn_index[depdata]
                                 depends.append(taskData.gettask_id(dep, taskname))
 
@@ -130,12 +131,13 @@ class RunQueue:
                 for idepend in idepends:
                     depid = int(idepend.split(":")[0])
                     if depid in taskData.build_targets:
+                        # Won't be in build_targets if ASSUME_PROVIDED
                         depdata = taskData.build_targets[depid][0]
-                        if depdata:
+                        if depdata is not None:
                             dep = taskData.fn_index[depdata]
                             depends.append(taskData.gettask_id(dep, idepend.split(":")[1]))
 
-                def add_recursive_build(depid):
+                def add_recursive_build(depid, depfnid):
                     """
                     Add build depends of depid to depends
                     (if we've not see it before)
@@ -146,23 +148,30 @@ class RunQueue:
                     dep_seen.append(depid)
                     if depid in taskData.build_targets:
                         depdata = taskData.build_targets[depid][0]
-                        if depdata:
+                        if depdata is not None:
                             dep = taskData.fn_index[depdata]
+                            idepends = []
                             # Need to avoid creating new tasks here
                             taskid = taskData.gettask_id(dep, taskname, False)
-                            if taskid:
+                            if taskid is not None:
                                 depends.append(taskid)
                                 fnid = taskData.tasks_fnid[taskid]
+                                idepends = taskData.tasks_idepends[taskid]
+                                #print "Added %s (%s) due to %s" % (taskid, taskData.fn_index[fnid], taskData.fn_index[depfnid])
                             else:
                                 fnid = taskData.getfn_id(dep)
                             for nextdepid in taskData.depids[fnid]:
                                 if nextdepid not in dep_seen:
-                                    add_recursive_build(nextdepid)
+                                    add_recursive_build(nextdepid, fnid)
                             for nextdepid in taskData.rdepids[fnid]:
                                 if nextdepid not in rdep_seen:
-                                    add_recursive_run(nextdepid)
+                                    add_recursive_run(nextdepid, fnid)
+                            for idepend in idepends:
+                                nextdepid = int(idepend.split(":")[0])
+                                if nextdepid not in dep_seen:
+                                    add_recursive_build(nextdepid, fnid)
 
-                def add_recursive_run(rdepid):
+                def add_recursive_run(rdepid, depfnid):
                     """
                     Add runtime depends of rdepid to depends
                     (if we've not see it before)
@@ -173,33 +182,44 @@ class RunQueue:
                     rdep_seen.append(rdepid)
                     if rdepid in taskData.run_targets:
                         depdata = taskData.run_targets[rdepid][0]
-                        if depdata:
+                        if depdata is not None:
                             dep = taskData.fn_index[depdata]
+                            idepends = []
                             # Need to avoid creating new tasks here
                             taskid = taskData.gettask_id(dep, taskname, False)
-                            if taskid:
+                            if taskid is not None:
                                 depends.append(taskid)
                                 fnid = taskData.tasks_fnid[taskid]
+                                idepends = taskData.tasks_idepends[taskid]
+                                #print "Added %s (%s) due to %s" % (taskid, taskData.fn_index[fnid], taskData.fn_index[depfnid])
                             else:
                                 fnid = taskData.getfn_id(dep)
                             for nextdepid in taskData.depids[fnid]:
                                 if nextdepid not in dep_seen:
-                                    add_recursive_build(nextdepid)
+                                    add_recursive_build(nextdepid, fnid)
                             for nextdepid in taskData.rdepids[fnid]:
                                 if nextdepid not in rdep_seen:
-                                    add_recursive_run(nextdepid)
+                                    add_recursive_run(nextdepid, fnid)
+                            for idepend in idepends:
+                                nextdepid = int(idepend.split(":")[0])
+                                if nextdepid not in dep_seen:
+                                    add_recursive_build(nextdepid, fnid)
 
 
                 # Resolve Recursive Runtime Depends
-                # Also includes all Build Depends (and their runtime depends)
+                # Also includes all thier build depends, intertask depends and runtime depends
                 if 'recrdeptask' in task_deps and taskData.tasks_name[task] in task_deps['recrdeptask']:
                     for taskname in task_deps['recrdeptask'][taskData.tasks_name[task]].split():
                         dep_seen = []
                         rdep_seen = []
+                        idep_seen = []
                         for depid in taskData.depids[fnid]:
-                            add_recursive_build(depid)
+                            add_recursive_build(depid, fnid)
                         for rdepid in taskData.rdepids[fnid]:
-                            add_recursive_run(rdepid)
+                            add_recursive_run(rdepid, fnid)
+                        for idepend in idepends:
+                            depid = int(idepend.split(":")[0])
+                            add_recursive_build(depid, fnid)
 
                 #Prune self references
                 if task in depends:
@@ -466,6 +486,18 @@ class RunQueue:
         def sigint_handler(signum, frame):
             raise KeyboardInterrupt
 
+        # Find any tasks with current stamps and remove them from the queue
+        for task1 in range(len(self.runq_fnid)):
+            task = self.prio_map[task1]
+            fn = self.taskData.fn_index[self.runq_fnid[task]]
+            taskname = self.runq_task[task]
+            if bb.build.stamp_is_current(taskname, self.dataCache, fn):
+                bb.msg.debug(2, bb.msg.domain.RunQueue, "Stamp current task %s (%s)" % (task, self.get_user_idstring(task)))
+                self.runq_running[task] = 1
+                self.task_complete(task)
+                self.stats.taskCompleted()
+                self.stats.taskSkipped()
+
         while True:
             task = self.get_next_task()
             if task is not None:
@@ -492,7 +524,8 @@ class RunQueue:
                     # signal.signal(signal.SIGINT, signal.SIG_IGN)
                     # Make the child the process group leader
                     os.setpgid(0, 0)
-                    sys.stdin = open('/dev/null', 'r')
+                    newsi = os.open('/dev/null', os.O_RDWR)
+                    os.dup2(newsi, sys.stdin.fileno())
                     self.cooker.configuration.cmd = taskname[3:]
                     try: 
                         self.cooker.tryBuild(fn, False)
